"""
Utility functions
"""
from .conf import nlf_settings


def coerce_bool(value):
    """Coerces any value to a boolean. String values are checked for the first letter
    and matched against the `FALSE_VALUES` setting.
    """
    if isinstance(value, str):
        return value[0].lower() not in nlf_settings.FALSE_VALUES
    return value



"""
Configuration module
"""
import warnings

try:
    from django.conf import settings as dj_settings
    from django.core.signals import setting_changed
except ImportError:
    dj_settings = object()
    setting_changed = None


DEFAULTS = {
    "QUERY_PARAM": "q",
    "PATH_SEPARATOR": ".",
    "EMPTY_VALUE": "EMPTY",
    "FALSE_VALUES": ("0", "f"),
}


DEPRECATED_SETTINGS = []


def deprecate(msg, level_modifier=0):
    warnings.warn(msg, DeprecationWarning, stacklevel=3 + level_modifier)


def is_callable(value):
    # check for callables, except types
    return callable(value) and not isinstance(value, type)


class Settings:
    def __getattr__(self, name):
        if name not in DEFAULTS:
            msg = "'%s' object has no attribute '%s'"
            raise AttributeError(msg % (self.__class__.__name__, name))

        value = self.get_setting(name)

        if is_callable(value):
            value = value()

        # Cache the result
        setattr(self, name, value)
        return value

    def get_setting(self, setting):
        django_setting = f"NLF_{setting}"

        if setting in DEPRECATED_SETTINGS and hasattr(dj_settings, django_setting):
            deprecate(f"The '{django_setting}' setting has been deprecated.")

        return getattr(dj_settings, django_setting, DEFAULTS[setting])

    def change_setting(self, setting, value, enter, **kwargs):
        if not setting.startswith("NLF_"):
            return
        setting = setting[4:]  # strip 'NLF_'

        # ensure a valid app setting is being overridden
        if setting not in DEFAULTS:
            return

        # if exiting, delete value to repopulate
        if enter:
            setattr(self, setting, value)
        else:
            delattr(self, setting)


nlf_settings = Settings()

if setting_changed is not None:
    setting_changed.connect(nlf_settings.change_setting)



#!/usr/bin/env python

from setuptools import setup

from django_nlf import __version__ as dnlf_version


with open('README.md') as desc:
    LONG_DESCRIPTION = desc.read()

with open('requirements.txt') as reqs:
    REQUIREMENTS = reqs.readlines()

setup(
    name='django-nlf',
    version=dnlf_version,
    description='Django Natural Language Filter package',
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author='Hodossy, Szabolcs',
    author_email='hodossy.szabolcs@gmail.com',
    maintainer='Hodossy, Szabolcs',
    maintainer_email='hodossy.szabolcs@gmail.com',
    url='https://github.com/hodossy/django-nlf',
    license='MIT',
    platforms='any',
    packages=['django_nlf'],
    keywords=['django', 'natural-Language', 'filtering'],
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Typing :: Typed',
    ],
    python_requires='>=3.7',
    install_requires=REQUIREMENTS,
)



"""
Django NLF types
"""
import typing
from dataclasses import dataclass, field
from enum import Enum, auto


class Lookup(Enum):
    """Enumeration of supported lookups."""

    EQUALS = auto()
    CONTAINS = auto()
    REGEX = auto()
    IN = auto()
    GT = auto()
    GTE = auto()
    LT = auto()
    LTE = auto()


class Operation(Enum):
    """Enumeration of operations."""

    AND = auto()
    OR = auto()


@dataclass(eq=True)
class Expression:
    """The most basic building block of the language."""

    lookup: Lookup
    field: str
    value: str
    exclude: bool


@dataclass()
class CustomFunction:
    """The interface for registering and calling custom functions."""

    name: str
    args: typing.Iterable[str] = field(default_factory=list)
    kwargs: typing.Mapping = field(default_factory=dict)


@dataclass()
class CompositeExpression:
    """Combines expressions (as operands) with an operator.
    Operands can be other composite expressions as well.
    """

    operation: Operation
    left: typing.Union[Expression, "CompositeExpression", CustomFunction]
    right: typing.Union[Expression, "CompositeExpression", CustomFunction]


ParsedExpression = typing.Union[Expression, CompositeExpression, CustomFunction]


class FunctionRole(Enum):
    """Enumeration of custom function roles"""

    FIELD = auto()
    VALUE = auto()
    EXPRESSION = auto()


@dataclass()
class FunctionMeta:
    """The metadata for custom functions that help verify their usage in the language."""

    role: FunctionRole = FunctionRole.VALUE
    models: typing.Iterable["django.db.models.Model"] = tuple()
